#! /usr/bin/env python

"""
Test case exhibiting @cached_property bug, where the getter is called
immediately when one of the dependencies changes, rather than waiting
until the value is actually requested.

Original Author: David Banas <capn.freako@gmail.com>

Original Date:   15 February 2015

Copyright (c) 2015 by David Banas; All rights reserved World wide.
"""

from traits.api      import HasTraits, Int, Property, Array, cached_property
from traitsui.api    import View, Item, Action, Handler
from numpy           import arange

class MyHandler(Handler):
    """This handler is instantiated by the View and handles user button clicks."""

    def do_print_y(self, info):
        print "y:", info.object.y
        print "z:", info.object.z

print_y = Action(name="Print y", action="do_print_y")
    
class CachedPropertyTester(HasTraits):
    x     = Int(2)
    y     = Property(Array, depends_on=['x'])
#    z = Property(Array, depends_on=['y'])
    z = Property(Array)

    @cached_property
    def _get_y(self):
        print "Just entered _get_y()."
        return arange(self.x)

    @cached_property
    def _get_z(self):
        print "Just entered _get_z()."
        y = self.y
        return y ** 2

    view = View(Item('x'), buttons = [print_y, "OK"], handler = MyHandler())

if __name__ == '__main__':
    CachedPropertyTester().configure_traits()

